/*
* vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Ecore_Evas.h>
#include <Ecore_Str.h>
#include <Edje.h>

#include "enna.h"
#include "enna_module.h"
#include "enna_list.h"
#include "enna_fm.h"
#include "enna_mediaplayer.h"
#include "enna_config.h"
#include "enna_volume_manager.h"
#include "enna_cdda.h"
#include "enna_metadata.h"

#define SMART_NAME "enna_module_music"
#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
#define INTERNAL_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if (!sd) return;

#define MUSIC_MODULE_STATE_MENU 0
#define MUSIC_MODULE_STATE_FM 1
#define MUSIC_MODULE_STATE_PLAYER 2
#define MUSIC_MODULE_STATE_CDDA 3

typedef struct _E_Smart_Data E_Smart_Data;

struct _E_Smart_Data
{
	Enna        *enna;
	Evas_Coord x, y, w, h;
	Evas_Object *edje;
	Evas_Object *obj;
	Evas_Object *switcher;
	Evas_Object *menu;
	Evas_Object *fm;
	Evas_Object *cdda;
	Evas_Object *cover;
	char *music_path;
	char *radio_path;
	unsigned int state;
};

/* local subsystem functions */
static void _switch_objects (E_Smart_Data * sd, Evas_Object * obj1,
							 Evas_Object * obj2);
static int _late_clear (void *data);
static void _update_mainmenu (E_Smart_Data * sd);
static void _e_smart_reconfigure (E_Smart_Data * sd);
static void _e_smart_add (Evas_Object * obj);
static void _e_smart_del (Evas_Object * obj);
static void _e_smart_move (Evas_Object * obj, Evas_Coord x, Evas_Coord y);
static void _e_smart_resize (Evas_Object * obj, Evas_Coord w, Evas_Coord h);
static void _e_smart_show (Evas_Object * obj);
static void _e_smart_hide (Evas_Object * obj);
static void _e_smart_color_set (Evas_Object * obj, int r, int g, int b,
								int a);
static void _e_smart_clip_set (Evas_Object * obj, Evas_Object * clip);
static void _e_smart_clip_unset (Evas_Object * obj);
static void _e_smart_init (void);

/* local subsystem globals */
static Evas_Smart *_e_smart = NULL;

/* externally accessible functions */
EAPI Evas_Object *
enna_module_music_add (Evas * evas)
{
	_e_smart_init ();
	return evas_object_smart_add (evas, _e_smart);
}

static void
_enna_module_music_init (E_Smart_Data * sd)
{
	
	sd->music_path = enna_config_get_conf_value_or_default ("music_module",
															"base_path", "/");
	sd->radio_path = enna_config_get_conf_value_or_default ("radio_module",
															"base_path", "/");
	
}


EAPI void
enna_module_music_process_event (Evas_Object * obj, enna_event event)
{
	API_ENTRY return;
	
	if (sd->state == MUSIC_MODULE_STATE_MENU)
		enna_list_process_event (sd->menu, event);
	else if (sd->state == MUSIC_MODULE_STATE_FM)
	{
		
		if ((event == enna_event_up || event == enna_event_down))
		{
			edje_object_signal_emit (sd->edje, "enna,state,infos,hide", "enna");
		}
		enna_fm_process_event (sd->fm, event);
	}
	else if (sd->state == MUSIC_MODULE_STATE_CDDA)
		enna_cdda_process_event (sd->cdda, event);
}

EAPI void
enna_module_music_focus_set (Evas_Object * obj, unsigned int focus)
{
	API_ENTRY return;
}

static void
_exit_fm_cb (void *data1, void *data2)
{
	E_Smart_Data *sd;
	
	sd = (E_Smart_Data *) data1;
	if (!sd)
		return;
	
	_switch_objects (sd, sd->fm, NULL);
	ecore_timer_add (1.0, _late_clear, sd->fm);
	sd->state = MUSIC_MODULE_STATE_MENU;
	_update_mainmenu (sd);
	
}

static void
_fm_change_path_cb (void *data1, char *path)
{
	E_Smart_Data *sd;
	
	sd = (E_Smart_Data *) data1;
	if (!sd)
		return;
	dbg("change path : %s\n", path);
	
	edje_object_signal_emit (sd->edje, "enna,state,path,hide",
							 "enna");
	edje_object_part_text_set (sd->edje, "enna.text.path",
							   (char*)basename(path));
	edje_object_signal_emit (sd->edje, "enna,state,path,show",
							 "enna");
	
}

static void
_exit_cdda_cb (void *data1, void *data2)
{
	E_Smart_Data *sd;
	
	sd = (E_Smart_Data *) data1;
	if (!sd)
		return;
	
	_switch_objects (sd, sd->cdda, NULL);
	ecore_timer_add (1.0, _late_clear, sd->cdda);
	sd->state = MUSIC_MODULE_STATE_MENU;
	_update_mainmenu (sd);
}


static void
_play_file_cb (void *data1, void *data2, char *filename)
{
	E_Smart_Data *sd;
	
	sd = (E_Smart_Data *) data1;
	
	if (!sd)
		return;

	enna_mediaplayer_play (sd->enna->mediaplayer, filename,
						   ENNA_MEDIA_TYPE_MUSIC_FILE, NULL);
	
}

static void
_hilight_file_cb (void *data1, void *data2, char *filename)
{
	E_Smart_Data *sd;
	Enna *enna;
	Enna_Metadata *metadata;
	long mb, kb;
	long m, s;
	char tmp[256];
	char tmp2[256];
	int w, h;
	
	
	sd = (E_Smart_Data *) data1;
	
	
	if (!sd)
		return;
	
	
	if (!filename)
		edje_object_signal_emit (sd->edje, "enna,state,infos,hide", "enna");
	else
	{
		
		edje_object_signal_emit (sd->edje, "enna,state,infos,hide", "enna");
		enna = evas_data_attach_get (evas_object_evas_get (sd->edje));
		metadata = enna_metadata_get (filename);
		edje_object_part_text_set (sd->edje, "enna.text.album",
								   metadata->album);
		edje_object_part_text_set (sd->edje, "enna.text.title",
								   metadata->title);
		edje_object_part_text_set (sd->edje, "enna.text.artist",
								   metadata->artist);
		edje_object_part_text_set (sd->edje, "enna.text.filename",
								   ecore_file_get_file (filename));
		
		edje_object_part_text_set (sd->edje, "enna.text.genre",
								   metadata->genre);
		edje_object_part_text_set (sd->edje, "enna.text.filetype",
								   metadata->filetype);
		mb = metadata->size / (1024 * 1024);
		kb = metadata->size / 1024 - (mb * 1024);
		snprintf (tmp, sizeof (tmp), "%li,%li MB", mb, kb);
		edje_object_part_text_set (sd->edje, "enna.text.size", tmp);
		m = metadata->duration / 60;
		s = metadata->duration - (m * 60);
		snprintf (tmp2, sizeof (tmp2), "%02li mn %02li sec", m, s);
		edje_object_part_text_set (sd->edje, "enna.text.duration", tmp2);
		
		
		if (metadata->cover_filename)
		{
			edje_object_part_unswallow (sd->edje, sd->cover);
			evas_object_hide (sd->cover);
			evas_object_del (sd->cover);
			sd->cover = evas_object_image_add (evas_object_evas_get (sd->edje));
			edje_object_part_swallow (sd->edje, "enna.cover.swallow",
									  sd->cover);
			edje_object_part_geometry_get (sd->edje, "enna.cover.swallow", NULL,
										   NULL, &w, &h);
			evas_object_image_file_set (sd->cover, metadata->cover_filename,
										NULL);
			evas_object_resize (sd->cover, w, h);
			evas_object_image_fill_set (sd->cover, 0, 0, w, h);
			evas_object_show (sd->cover);
		}
		else
		{
			edje_object_part_unswallow (sd->edje, sd->cover);
			evas_object_hide (sd->cover);
			evas_object_del (sd->cover);
			/*sd->cover = edje_object_add(evas_object_evas_get(sd->obj));
			edje_object_file_set(sd->edje, enna_config_theme_get(), "unknown_cover");
			
			edje_object_part_geometry_get(sd->edje, "enna.cover.swallow", NULL, NULL, &w, &h);
			edje_extern_object_min_size_set(sd->cover, w, h);
			edje_object_part_swallow(sd->edje, "enna.cover.swallow", sd->cover);
			evas_object_resize(sd->cover, w, h);
			evas_object_show(sd->cover); */
		}
		edje_object_signal_emit (sd->edje, "enna,state,infos,show", "enna");
	}
	
	
}

static void
_play_radio_cb (void *data1, void *data2, char *filename)
{
	E_Smart_Data *sd;	
	sd = (E_Smart_Data *) data1;
	
	if (!sd)
		return;
}

static void
_play_track_cb (void *data1, void *data2, int selected)
{
	E_Smart_Data *sd;
	int nb_tracks = 0;
	char *device = NULL;
	char tmp[4096];
	
	sd = (E_Smart_Data *) data1;
	
	if (!sd)
		return;
	
	nb_tracks = enna_cdda_nbtracks_get (sd->cdda);
	device = enna_cdda_device_get (sd->cdda);
	
	sprintf (tmp, "cdda://%s#%d#%d", device, nb_tracks, selected);
	enna_mediaplayer_play (sd->enna->mediaplayer, tmp, ENNA_MEDIA_TYPE_MUSIC_CDDA,
						   NULL);
	
}
static void
_listen_to_webradio_cb (void *data, void *data2)
{
	E_Smart_Data *sd;
	char *root_path;
	
	sd = (E_Smart_Data *) data;

	root_path = (char *) data2;
	if (!root_path)
		return;
	
	//dbg("Listen to Music cb\n");
	//edje_object_part_unswallow(sd->edje, sd->menu);
	sd->fm = enna_fm_add (evas_object_evas_get (sd->edje));
	//edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->fm);
	evas_object_show (sd->fm);
	evas_object_hide (sd->menu);
	_switch_objects (sd, sd->menu, sd->fm);
	ecore_timer_add (1.0, _late_clear, sd->menu);
	sd->state = MUSIC_MODULE_STATE_FM;
	enna_fm_filter_set (sd->fm, "radio");
	enna_fm_root_set (sd->fm, root_path);
	edje_object_signal_emit (sd->edje, "enna,state,path,hide",
							 "enna");
	edje_object_part_text_set (sd->edje, "enna.text.path",
							   "Web Radios");
	edje_object_signal_emit (sd->edje, "enna,state,path,show",
							 "enna");
	enna_fm_exe_file_cb_set (sd->fm, _play_radio_cb, sd, NULL);
	enna_fm_exit_cb_set (sd->fm, _exit_fm_cb, sd, NULL);
	enna_fm_exit_cb_set (sd->fm, _exit_fm_cb, sd, NULL);
	
}

static void
_listen_to_cdda_cb (void *data, void *data2)
{
	E_Smart_Data *sd;
	char *device;
	sd = (E_Smart_Data *) data;
	
	device = (char *) data2;
	//edje_object_part_unswallow(sd->edje, sd->menu);
	sd->cdda = enna_cdda_add (evas_object_evas_get (sd->edje));
	//edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->cdda);
	evas_object_show (sd->cdda);
	evas_object_hide (sd->menu);
	_switch_objects (sd, sd->menu, sd->cdda);
	ecore_timer_add (1.0, _late_clear, sd->menu);
	enna_cdda_device_set (sd->cdda, device);
	//dbg("OK\n");
	sd->state = MUSIC_MODULE_STATE_CDDA;
	enna_cdda_exe_track_cb_set (sd->cdda, _play_track_cb, sd, NULL);
	enna_cdda_exit_cb_set (sd->cdda, _exit_cdda_cb, sd, NULL);
	//dbg("OK music module\n");
}

static void
_listen_to_ipod_cb (void *data, void *data2)
{
	dbg ("Listen iPOD\n");
}

static void
_listen_to_music_cb (void *data, void *data2)
{
	E_Smart_Data *sd;
	char *root_path;
	
	sd = (E_Smart_Data *) data;
	
	root_path = (char *) data2;
	if (!root_path)
		return;
	
	//dbg("Listen to Music cb\n");
	//edje_object_part_unswallow(sd->edje, sd->menu);
	sd->fm = enna_fm_add (evas_object_evas_get (sd->edje));
	//edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->fm);
	evas_object_show (sd->fm);
	evas_object_hide (sd->menu);
	_switch_objects (sd, sd->menu, sd->fm);
	ecore_timer_add (1.0, _late_clear, sd->menu);
	sd->state = MUSIC_MODULE_STATE_FM;
	enna_fm_filter_set (sd->fm, "music");
	enna_fm_root_set (sd->fm, root_path);
	edje_object_signal_emit (sd->edje, "enna,state,path,hide",
							 "enna");
	edje_object_part_text_set (sd->edje, "enna.text.path",
							   "Digital Corner");
	edje_object_signal_emit (sd->edje, "enna,state,path,show",
							 "enna");
	enna_fm_exe_file_cb_set (sd->fm, _play_file_cb, sd, NULL);
	enna_fm_hilight_file_cb_set (sd->fm, _hilight_file_cb, sd, NULL);
	enna_fm_exit_cb_set (sd->fm, _exit_fm_cb, sd, NULL);
	enna_fm_change_path_cb_set (sd->fm, _fm_change_path_cb, sd);
}

static void
_e_smart_reconfigure (E_Smart_Data * sd)
{
	evas_object_move (sd->edje, sd->x, sd->y);
	evas_object_resize (sd->edje, sd->w, sd->h);
}

static void
_update_mainmenu (E_Smart_Data * sd)
{
	Evas_Object *icon;
	Volume *v;
	
	edje_object_signal_emit (sd->edje, "enna,state,path,hide",
							 "enna");
	edje_object_part_text_set (sd->edje, "enna.text.path",
							   "Listen to Music");
	edje_object_signal_emit (sd->edje, "enna,state,path,show",
							 "enna");
	if (sd->state != MUSIC_MODULE_STATE_MENU)
		return;
	
	evas_object_del (sd->menu);
	sd->menu = enna_list_add (evas_object_evas_get (sd->obj));
	
	v = ecore_list_first_goto (sd->enna->volumes);
	while ((v = ecore_list_next (sd->enna->volumes)))
	{
		switch (v->type)
		{
			case DEV_AUDIO_CD:
			{
				dbg ("%s\n", v->label);
				icon = edje_object_add (evas_object_evas_get (sd->obj));
				edje_object_file_set (icon, enna_config_theme_get (),
									  "icon_cdda");
				enna_list_append (sd->menu, icon, "from Audio CD",
								  0, _listen_to_cdda_cb, NULL, sd, v->device);
				break;
			}
			case DEV_DATA_CD:
			{
				char tmp[4096];
				sprintf (tmp, "from CDROM (%s)", v->label);
				icon = edje_object_add (evas_object_evas_get (sd->obj));
				edje_object_file_set (icon, enna_config_theme_get (),
									  "icon_cdrom");
				evas_object_show (icon);
				enna_list_append (sd->menu, icon, tmp,
								  0, _listen_to_music_cb, NULL, sd,
								  v->mount_point);
				break;
			}
			case DEV_DATA_DVD:
			{
				char tmp[4096];
				sprintf (tmp, "from DVDROM (%s)", v->label);
				icon = edje_object_add (evas_object_evas_get (sd->obj));
				edje_object_file_set (icon, enna_config_theme_get (),
									  "icon_dvdrom");
				evas_object_show (icon);
				enna_list_append (sd->menu, icon, tmp,
								  0, _listen_to_music_cb, NULL, sd,
								  v->mount_point);
				break;
			}
			case DEV_USBDISK:
			{
				char tmp[4096];
				sprintf (tmp, "from USBDisk (%s)", v->label);
				icon = edje_object_add (evas_object_evas_get (sd->obj));
				edje_object_file_set (icon, enna_config_theme_get (),
									  "icon_usbdisk");
				evas_object_show (icon);
				enna_list_append (sd->menu, icon, tmp,
								  0, _listen_to_music_cb, NULL, sd,
								  v->mount_point);
				break;
			}
			case DEV_IPOD_DEVICE:
			{
				//char tmp[4096];
				//sprintf(tmp, "Listen Music on USBDisk (%s)", dev->volume_label);
				icon = edje_object_add (evas_object_evas_get (sd->obj));
				edje_object_file_set (icon, enna_config_theme_get (),
									  "icon_ipod");
				//evas_object_show(icon);
				enna_list_append (sd->menu, icon, _("Listen to iPod"),
								  0, _listen_to_ipod_cb, NULL, sd,
								  v->mount_point);
				break;
			}
			default:
				break;
		}
	}
	
	
	icon = edje_object_add (evas_object_evas_get (sd->obj));
	edje_object_file_set (icon, enna_config_theme_get (), "icon_hd");
	//evas_object_show(icon);
	enna_list_append (sd->menu, icon, "from Digital Corner", 0,
					  _listen_to_music_cb, NULL, sd, sd->music_path);
	icon = edje_object_add (evas_object_evas_get (sd->obj));
	edje_object_file_set (icon, enna_config_theme_get (), "icon_radio");
	//evas_object_show(icon);
	enna_list_append (sd->menu, icon, "from Web Radios", 0,
					  _listen_to_webradio_cb, NULL, sd, sd->radio_path);
	enna_list_selected_set (sd->menu, 0);
	_switch_objects (sd, NULL, sd->menu);
	//edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->menu);
	
}

static void
_switch_objects (E_Smart_Data * sd, Evas_Object * obj1, Evas_Object * obj2)
{
	if (!sd || (!obj1 && !obj2))
		return;
	
	if ((obj1 && obj2))
	{
		edje_object_part_unswallow (sd->switcher, obj1);
		edje_object_part_swallow (sd->switcher, "enna.switcher.swallow2", obj1);
		edje_object_signal_emit (sd->switcher, "enna,swallow2,default,now",
								 "enna");
		edje_object_signal_emit (sd->switcher, "enna,swallow1,state1,now",
								 "enna");
		edje_object_part_swallow (sd->switcher, "enna.switcher.swallow1", obj2);
		edje_object_signal_emit (sd->switcher, "enna,swallow1,default", "enna");
		edje_object_signal_emit (sd->switcher, "enna,swallow2,state2", "enna");
		evas_object_hide (obj1);
	}
	else if (!obj2)
	{
		edje_object_part_unswallow (sd->switcher, obj1);
		edje_object_part_swallow (sd->switcher, "enna.switcher.swallow2", obj1);
		edje_object_signal_emit (sd->switcher, "enna,swallow2,default,now",
								 "enna");
		edje_object_signal_emit (sd->switcher, "enna,swallow2,state2", "enna");
	}
	else if (!obj1)
	{
		edje_object_signal_emit (sd->switcher, "enna,swallow1,state1,now",
								 "enna");
		edje_object_part_swallow (sd->switcher, "enna.switcher.swallow1", obj2);
		edje_object_signal_emit (sd->switcher, "enna,swallow1,default", "enna");
		evas_object_show (obj2);
	}
}

static int
_late_clear (void *data)
{
	Evas_Object *obj;
	
	obj = (Evas_Object *) data;
	
	evas_object_hide (obj);
	evas_object_del (obj);
	
	return 0;
}

static int
_device_add_event (void *data, int type, void *ev)
{
	E_Smart_Data *sd;

	sd = (E_Smart_Data *) data;
	
	_update_mainmenu (sd);
	return 1;
	
}
static void
_e_smart_event_mouse_down (void *data, Evas * e, Evas_Object * obj,
						   void *event_info)
{
	Evas_Event_Mouse_Down *ev;
	ev = event_info;
}

static void
_e_smart_event_mouse_up (void *data, Evas * e, Evas_Object * obj,
						 void *event_info)
{
	Evas_Event_Mouse_Up *ev;
	ev = event_info;
}

static void
_e_smart_add (Evas_Object * obj)
{
	E_Smart_Data *sd;
	Enna *enna;
	sd = calloc (1, sizeof (E_Smart_Data));
	if (!sd)
		return;
	evas_object_smart_data_set (obj, sd);
	_enna_module_music_init (sd);
	
	sd->obj = obj;
	sd->x = 0;
	sd->y = 0;
	sd->w = 0;
	sd->h = 0;
	sd->edje = edje_object_add (evas_object_evas_get (obj));
	edje_object_file_set (sd->edje, enna_config_theme_get (),
						  "enna/module/music");
	if (!sd->edje)
		dbg ("NOT EDJE\n");
	
	enna = evas_data_attach_get (evas_object_evas_get (sd->edje));
	sd->enna = enna;
	
	sd->switcher = edje_object_add (evas_object_evas_get (obj));
	edje_object_file_set (sd->switcher, enna_config_theme_get (), "enna/switcher");
	
	evas_object_smart_member_add (sd->switcher, obj);
	evas_object_show (sd->switcher);
	edje_object_part_swallow (sd->edje, "enna.switcher.swallow", sd->switcher);
	
	evas_object_event_callback_add (obj, EVAS_CALLBACK_MOUSE_DOWN,
									_e_smart_event_mouse_down, NULL);
	evas_object_event_callback_add (obj, EVAS_CALLBACK_MOUSE_UP,
									_e_smart_event_mouse_up, NULL);
	
	evas_object_smart_member_add (sd->edje, obj);
	evas_object_show (sd->edje);
	evas_object_propagate_events_set (obj, 0);
	
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_AUDIO_CD_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_DATA_CD_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_DATA_DVD_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_USBDISK_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_CAMERA_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_IPOD_ADD],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_AUDIO_CD_REMOVE],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_DATA_CD_REMOVE],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_DATA_DVD_REMOVE],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_USBDISK_REMOVE],
							 _device_add_event, sd);
	ecore_event_handler_add (sd->enna->events[ENNA_EVENT_DEV_CAMERA_REMOVE],
							 _device_add_event, sd);
	sd->state = MUSIC_MODULE_STATE_MENU;
	
	_update_mainmenu (sd);
	sd->cover = NULL;

	evas_object_show (sd->menu);
}

static void
_e_smart_del (Evas_Object * obj)
{
	INTERNAL_ENTRY;
	evas_object_del (sd->edje);
	evas_object_del (sd->switcher);
	if (sd->music_path)
		free (sd->music_path);
	evas_object_del (sd->cdda);
	evas_object_del (sd->fm);
	evas_object_del (sd->menu);
	
	free (sd);
}

static void
_e_smart_move (Evas_Object * obj, Evas_Coord x, Evas_Coord y)
{
	INTERNAL_ENTRY;
	if ((sd->x == x) && (sd->y == y))
		return;
	sd->x = x;
	sd->y = y;
	_e_smart_reconfigure (sd);
}

static void
_e_smart_resize (Evas_Object * obj, Evas_Coord w, Evas_Coord h)
{
	INTERNAL_ENTRY;
	
	if ((sd->w == w) && (sd->h == h))
		return;
	sd->w = w;
	sd->h = h;
	_e_smart_reconfigure (sd);
	
}

static void
_e_smart_show (Evas_Object * obj)
{
	INTERNAL_ENTRY;
	evas_object_show (sd->edje);
}

static void
_e_smart_hide (Evas_Object * obj)
{
	INTERNAL_ENTRY;
	evas_object_hide (sd->edje);
}

static void
_e_smart_color_set (Evas_Object * obj, int r, int g, int b, int a)
{
	INTERNAL_ENTRY;
	evas_object_color_set (sd->edje, r, g, b, a);
}

static void
_e_smart_clip_set (Evas_Object * obj, Evas_Object * clip)
{
	INTERNAL_ENTRY;
	evas_object_clip_set (sd->edje, clip);
}

static void
_e_smart_clip_unset (Evas_Object * obj)
{
	INTERNAL_ENTRY;
	evas_object_clip_unset (sd->edje);
}

/* never need to touch this */

static void
_e_smart_init (void)
{
	if (_e_smart)
		return;
	static const Evas_Smart_Class sc =
	{
		SMART_NAME,
		EVAS_SMART_CLASS_VERSION,
		_e_smart_add,
		_e_smart_del,
		_e_smart_move,
		_e_smart_resize,
		_e_smart_show,
		_e_smart_hide,
		_e_smart_color_set,
		_e_smart_clip_set,
		_e_smart_clip_unset,
		NULL
	};
	_e_smart = evas_smart_class_new(&sc);
}
